package com.ruoyi.framework.aspectj.scop;


import com.ruoyi.framework.annotation.Dynamic;
import java.util.LinkedList;
import java.util.List;

/**
 * {@link Dynamic} 注解的 Context 上下文
 *
 * @author 芋道源码
 */
public class DataPermissionContextHolder {

    /**
     * 使用 List 的原因，可能存在方法的嵌套调用
     */
    // private static final ThreadLocal<LinkedList<Dynamic>> DATA_PERMISSIONS =
    //         TransmittableThreadLocal.withInitial(LinkedList::new);
    private static final ThreadLocal<LinkedList<Dynamic>> DATA_PERMISSIONS =
            ThreadLocal.withInitial(LinkedList::new);

    /**
     * 获得当前的 Dynamic 注解
     *
     * @return Dynamic 注解
     */
    public static Dynamic get() {
        return DATA_PERMISSIONS.get().peekLast();
    }

    /**
     * 入栈 Dynamic 注解
     *
     * @param dataPermission Dynamic 注解
     */
    public static void add(Dynamic dataPermission) {
        DATA_PERMISSIONS.get().addLast(dataPermission);
    }

    /**
     * 出栈 Dynamic 注解
     *
     * @return Dynamic 注解
     */
    public static Dynamic remove() {
        Dynamic dataPermission = DATA_PERMISSIONS.get().removeLast();
        // 无元素时，清空 ThreadLocal
        if (DATA_PERMISSIONS.get().isEmpty()) {
            DATA_PERMISSIONS.remove();
        }
        return dataPermission;
    }

    /**
     * 获得所有 Dynamic
     *
     * @return Dynamic 队列
     */
    public static List<Dynamic> getAll() {
        return DATA_PERMISSIONS.get();
    }

    /**
     * 清空上下文
     *
     * 目前仅仅用于单测
     */
    public static void clear() {
        DATA_PERMISSIONS.remove();
    }

}
