package com.ruoyi.project.business.mapper;

import java.util.List;
import java.util.Set;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.project.business.domain.TcAccountContact;
import com.ruoyi.project.business.domain.vo.account.ExistContactDto;

/**
 * 联系人Mapper接口
 * 
 * @author ruoyi
 * @date 2024-04-12
 */
public interface TcAccountContactMapper extends BaseMapper<TcAccountContact>
{
    /**
     * 查询联系人
     * 
     * @param contactId 联系人主键
     * @return 联系人
     */
    public TcAccountContact selectTcAccountContactByContactId(Long contactId);

    /**
     * 查询相关联系人
     * 
     * @param accountId 客户ID
     * @return 联系人集合
     */
    public List<TcAccountContact> selectContactlistByAccountId(Long accountId);

    /**
     * 查询联系人列表
     *
     * @param tcAccountContact 联系人
     * @return 联系人集合
     */
    public List<TcAccountContact> selectTcAccountContactList(TcAccountContact tcAccountContact);

    /**
     * 新增联系人
     * 
     * @param tcAccountContact 联系人
     * @return 结果
     */
    public int insertTcAccountContact(TcAccountContact tcAccountContact);

    /**
     * 修改联系人
     * 
     * @param tcAccountContact 联系人
     * @return 结果
     */
    public int updateTcAccountContact(TcAccountContact tcAccountContact);

    /**
     * 删除联系人
     * 
     * @param contactId 联系人主键
     * @return 结果
     */
    public int deleteTcAccountContactByContactId(Long contactId);

    /**
     * 批量删除联系人
     * 
     * @param contactIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTcAccountContactByContactIds(Long[] contactIds);

    /**
     * 查询未关联客户联系人
     *
     * @param tcAccountContact 联系人
     * @return 结果
     */
    List<TcAccountContact> getUnRelatedContact(TcAccountContact tcAccountContact);

    void deleteTcAccountContactByAccountId(Long accountId);

    void deleteTcAccountContactByAccountIds(Long[] accountIds);

    /**
     * 根据联系人ID查询联系人 name和客户name
     * 自行校验ids是否为空
     */
    List<ExistContactDto> selectExistContactAccount(Set<Long> contactIds, String permissionChar);
}
